      subroutine hdesim2(u,w,visc,dx,dz,dxs,dzs,
     $     dxi,dzi,dxsi,dzsi,dtrm,relu,relw,relp,jstas,jends)
      use para4common
************************************************************************
*     solve two-dimensional hydrodynamic equations
*     by using SIMPLER method
*     harmonic equations are solved by SSOR+CG method
*     by calling ssorcg subroutines
************************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,ij,ich,nsmp,maxsmp,k
      parameter(maxsmp=9999)
      real*8 u(iax,jax),w(iax,jax),dtrm(iax,jax)
      real*8 dx(iax),dxs(iax),dz(jax),dzs(jax)
      real*8 dxi(iax),dxsi(iax),dzi(jax),dzsi(jax)
      real*8 visc(iax,jax),vcc(iax,jax),vgx(iax,jax),vgz(iax,jax)
c
      real*8 zansa,zansatmp,epssim,errors(maxsmp)
      parameter(epssim=1.d-10)
      real*8 relu,relw,relp
      real*8 arelu,arelw,arelp,arelu1,arelw1,arelp1
      real*8 buoy(iax,jax)
      real*8 apx1(5,iax,jax),apz1(5,iax,jax),app1(5,iax,jax)
      real*8 apx2(5,iax,jax),apz2(5,iax,jax),app2(5,iax,jax)
      real*8 dudp1(iax,jax),dwdp1(iax,jax)
      real*8 pstar(iax,jax),pdash(iax,jax)
      real*8 uhat(iax,jax),ustar(iax,jax),udash(iax,jax),uwk(iax,jax)
      real*8 what(iax,jax),wstar(iax,jax),wdash(iax,jax),wwk(iax,jax)
c
c ------------------------------------
c     variables for SSOR-CG solver
c ------------------------------------
c
      integer nitr
      real*8 eitr,epsres
      parameter(epsres=epssim*1.d-5)
c
      integer mxx,mzx,mx2x,mz2x,nmx
      parameter(mxx=iax-3,mzx=jax-2)
      parameter(nmx=mxx*mzx,mx2x=mxx+2,mz2x=mzx+2)
      real*8 amtx(5,mx2x,mz2x),bvcx(mx2x,mz2x)
     $     ,xvcx(mx2x,mz2x)
      real*8 wkrx(mx2x,mz2x),wkpx(mx2x,mz2x)
     $     ,wkrux(mx2x,mz2x),wkapx(mx2x,mz2x)
c
      integer mxz,mzz,mx2z,mz2z,nmz
      parameter(mxz=iax-2,mzz=jax-3)
      parameter(nmz=mxz*mzz,mx2z=mxz+2,mz2z=mzz+2)
      real*8 amtz(5,mx2z,mz2z),bvcz(mx2z,mz2z)
     $     ,xvcz(mx2z,mz2z)
      real*8 wkrz(mx2z,mz2z),wkpz(mx2z,mz2z)
     $     ,wkruz(mx2z,mz2z),wkapz(mx2z,mz2z)
c
      integer mxp,mzp,mx2p,mz2p,nmp
      parameter(mxp=iax-2,mzp=jax-2)
      parameter(nmp=mxp*mzp,mx2p=mxp+2,mz2p=mzp+2)
      real*8 amtp(5,mx2p,mz2p),bvcp(mx2p,mz2p)
     $     ,xvcp(mx2p,mz2p)
      real*8 wkrp(mx2p,mz2p),wkpp(mx2p,mz2p)
     $     ,wkrup(mx2p,mz2p),wkapp(mx2p,mz2p)
c
      integer jstas(0:maxprocs-1),jends(0:maxprocs-1)
      integer jsta,jend,jsta2,jsta3,jend1
      integer jstau(0:maxprocs-1),jendu(0:maxprocs-1)
      integer jstaw(0:maxprocs-1),jendw(0:maxprocs-1)
      integer jstap(0:maxprocs-1),jendp(0:maxprocs-1)
c
      integer is1,is2,ir1,ir2
      real*8 bufs1(iax),bufs2(iax),bufr1(iax),bufr2(iax)
c
c ------------------------------------------
c     define communication configuration
c ------------------------------------------
c
      jsta = jstas(myrank)
      jend = jends(myrank)
      jsta2 = max(2,jsta)
      jsta3 = max(3,jsta)
      jend1 = min(jax-1,jend)
c
c -----------------------
c     set work arrays
c -----------------------
c
      do j = jsta,jend
         do i = 1,iax
            uwk(i,j) = u(i,j)
            wwk(i,j) = w(i,j)
         enddo
      enddo
c
c ------------------------------------------
c     calculate viscosity at grid points
c ------------------------------------------
c
      do j = jsta2,jend1
         do i = 2,iax-1
            vcc(i,j) = visc(i,j)
         enddo
      enddo
      if (myrank.eq.0) then
         do i = 2,iax-1
            vcc(i,1) = vcc(i,2)
         enddo
      endif
      if (myrank.eq.nprocs-1) then
         do i = 2,iax-1
            vcc(i,jax) = vcc(i,jax-1)
         enddo
      endif
      do j = jsta,jend
         vcc(1,j) = vcc(2,j)
         vcc(iax,j) = vcc(iax-1,j)
      enddo
c
      call myshift(vcc,jstas,jends)
c
      call calvisg(vcc,vgx,vgz,dx,dz,dxs,dzs,dxi,dzi,dxsi,dzsi
     $     ,jstas,jends)
c      call mypartialshift(vgx,jsta,jend,2)
c
c     calculate buoyancy at grid points
c
      call mypartialshift(dtrm,jsta,jend,1)
      do j = jsta2,jend
         do i = 2,iax-1
            buoy(i,j) = 0.5d0*dx(i)
     $           *(dtrm(i,j-1)*dz(j-1)+dtrm(i,j)*dz(j))
         enddo
      enddo
c
c ------------------------------------------
c     calculate coefficients for u and w
c ------------------------------------------
c
      do j = jsta2,jend1
         do i = 2,iax
            apx1(1,i,j) = vgx(i,j)*dxs(i)*dzsi(j)
            apx1(2,i,j) = 2.d0*vcc(i-1,j)*dxi(i-1)*dz(j)
            apx1(4,i,j) = 2.d0*vcc(i,j)*dxi(i)*dz(j)
            apx1(5,i,j) = vgx(i,j+1)*dxs(i)*dzsi(j+1)
            apx1(3,i,j) =
     $           apx1(1,i,j)+apx1(2,i,j)+apx1(4,i,j)+apx1(5,i,j)
         enddo
      enddo
c
      do j = jsta2,jend
         do i = 2,iax-1
            apz1(1,i,j) = 2.d0*vcc(i,j-1)*dx(i)*dzi(j-1)
            apz1(2,i,j) = vgz(i,j)*dxsi(i)*dzs(j)
            apz1(4,i,j) = vgz(i+1,j)*dxsi(i+1)*dzs(j)
            apz1(5,i,j) = 2.d0*vcc(i,j)*dx(i)*dzi(j)
            apz1(3,i,j) =
     $           apz1(1,i,j)+apz1(2,i,j)+apz1(4,i,j)+apz1(5,i,j)
         enddo
      enddo
c
c -----------------------------------------------------
c     calculate correction coefficients for u and w
c -----------------------------------------------------
c
      do j = jsta2,jend1
         do i = 2,iax
            dudp1(i,j) = dz(j)/apx1(3,i,j)
         enddo
      enddo
c
      do j = jsta2,jend
         do i = 2,iax-1
            dwdp1(i,j) = dx(i)/apz1(3,i,j)
         enddo
      enddo
c
c -----------------------------------
c     calculate coefficient for p
c -----------------------------------
c
      call mypartialshift(dwdp1,jsta,jend,2)
      do j = jsta2,jend1
         do i = 2,iax-1
            app1(1,i,j) = dx(i)*dwdp1(i,j)
            app1(2,i,j) = dz(j)*dudp1(i,j)
            app1(4,i,j) = dz(j)*dudp1(i+1,j)
            app1(5,i,j) = dx(i)*dwdp1(i,j+1)
            app1(3,i,j) =
     $           app1(1,i,j)+app1(2,i,j)+app1(4,i,j)+app1(5,i,j)
         enddo
      enddo
c
c ---------------------------------
c     apply boundary conditions
c ---------------------------------
c
      do j = jsta,jend
         do i = 1,iax
            do k = 1,5
               apx2(k,i,j) = apx1(k,i,j)
               apz2(k,i,j) = apz1(k,i,j)
               app2(k,i,j) = app1(k,i,j)
            enddo
         enddo
      enddo
c
c     bottom boundary (j=2)
c     impermeable, free slip
c
      if (myrank.eq.0) then
c
         do i = 2,iax
            apx2(3,i,2) = apx2(3,i,2) - apx2(1,i,2)
            apx2(1,i,2) = 0.d0
         enddo
c
         do i = 2,iax-1
            apz2(1,i,3) = 0.d0
         enddo
c
         do i = 2,iax-1
            app2(3,i,2) = app2(3,i,2) - app2(1,i,2)
            app2(1,i,2) = 0.d0
         enddo
c
         do i = 2,iax-1
            dwdp1(i,2) = 0.d0
         enddo
c
      endif
c
c     top boundary (j=jax)
c     impermeable, free slip
c
      if (myrank.eq.nprocs-1) then
c
         do i = 2,iax-1
            apx2(3,i,jax-1) = apx2(3,i,jax-1) - apx2(5,i,jax-1)
            apx2(5,i,jax-1) = 0.d0
         enddo
c
         do i = 2,iax-1
            apz2(5,i,jax-1) = 0.d0
         enddo
c
         do i = 2,iax-1
            app2(3,i,jax-1) = app2(3,i,jax-1) - app2(5,i,jax-1)
            app2(5,i,jax-1) = 0.d0
         enddo
c
         do i = 2,iax-1
            dwdp1(i,jax) = 0.d0
         enddo
c
      endif
c
c     left-side wall (i=2)
c     impermeable free slip
c
      do j = jsta2,jend1
         apx2(2,3,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         apz2(3,2,j) = apz2(3,2,j) - apz2(2,2,j)
         apz2(2,2,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         app2(3,2,j) = app2(3,2,j) - app2(2,2,j)
         app2(2,2,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         dudp1(2,j) = 0.d0
      enddo
c
c     right-side wall (i=iax)
c     impermeable free slip
c
      do j = jsta2,jend1
         apx2(4,iax-1,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         apz2(3,iax-1,j) = apz2(3,iax-1,j) - apz2(4,iax-1,j)
         apz2(4,iax-1,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         app2(3,iax-1,j) = app2(3,iax-1,j) - app2(4,iax-1,j)
         app2(4,iax-1,j) = 0.d0
      enddo
c
      do j = jsta2,jend1
         dudp1(iax,j) = 0.d0
      enddo
c
c     to avoid singularity set p(2,2)=0
c
      if (myrank.eq.0) then
c
         app2(1,2,2) = 0.d0
         app2(2,2,2) = 0.d0
         app2(4,2,2) = 0.d0
         app2(5,2,2) = 0.d0
         app2(1,2,3) = 0.d0
         app2(2,3,2) = 0.d0
c
      endif
c
c -----------------------------------------
c     set/decompose coefficient matrice
c -----------------------------------------
c
c     ---- x ----
c
      do i = 0,nprocs-1
         jstau(i) = jstas(i)
         jendu(i) = jends(i)
      enddo
c
      do j = jsta2,jend1
         do i = 3,iax-1
            amtx(1,i-1,j) = -apx2(1,i,j)
            amtx(2,i-1,j) = -apx2(2,i,j)
            amtx(3,i-1,j) = apx2(3,i,j)
            amtx(4,i-1,j) = -apx2(4,i,j)
            amtx(5,i-1,j) = -apx2(5,i,j)
         enddo
      enddo
check_begin
c      write(50+myrank,*)"=== Ax ==="
c      do j = 2,nz+1
c         do i = 3,nx+1
c            write(50+myrank,'(2i3,5e12.4)')i,j,(amtx(k,i-1,j),k=1,5)
c         enddo
c      enddo
check_end
c
c     ---- z ----
c
      do i = 0,nprocs-1
         jstaw(i) = jstas(i)-1
         jendw(i) = jends(i)-1
      enddo
c
      do j = jsta3,jend1
         do i = 2,iax-1
            amtz(1,i,j-1) = -apz2(1,i,j)
            amtz(2,i,j-1) = -apz2(2,i,j)
            amtz(3,i,j-1) = apz2(3,i,j)
            amtz(4,i,j-1) = -apz2(4,i,j)
            amtz(5,i,j-1) = -apz2(5,i,j)
         enddo
      enddo
check_begin
c      write(50+myrank,*)"=== Az ==="
c      do j = 3,nz+1
c         do i = 2,nx+1
c            write(50+myrank,'(2i3,5e12.4)')i,j,(amtz(k,i,j-1),k=1,5)
c         enddo
c      enddo
check_end
c
c     ---- p ----
c
      do i = 0,nprocs-1
         jstap(i) = jstas(i)
         jendp(i) = jends(i)
      enddo
c
      do j = jsta2,jend1
         do i = 2,iax-1
            amtp(1,i,j) = -app2(1,i,j)
            amtp(2,i,j) = -app2(2,i,j)
            amtp(3,i,j) = app2(3,i,j)
            amtp(4,i,j) = -app2(4,i,j)
            amtp(5,i,j) = -app2(5,i,j)
         enddo
      enddo
check_begin
c      write(50+myrank,*)"=== Ap ==="
c      do j = 2,nz+1
c         do i = 2,nx+1
c            write(50+myrank,'(2i3,5e12.4)')i,j,(amtp(k,i,j),k=1,5)
c         enddo
c      enddo
check_end
c
c //////////////////////////
c     simpler loop start
c //////////////////////////
c
c -----------------------
c     set work arrays
c -----------------------
c
      do j = jsta,jend
         do i = 1,iax
            uwk(i,j) = u(i,j)
            wwk(i,j) = w(i,j)
            ustar(i,j) = u(i,j)
            wstar(i,j) = w(i,j)
            pstar(i,j) = 0.d0
         enddo
      enddo
c
c     relaxation is not applied for 1st iteration loop
c
      arelu = 1.d0
      arelw = 1.d0
      arelp = 1.d0
      arelu1 = 1.d0-arelu
      arelw1 = 1.d0-arelw
      arelp1 = 1.d0-arelp
c
      ich = 1
      do nsmp = 1,maxsmp
c
c ----------------------------------------------
c     step 2: calculate dummy velocity field
c ----------------------------------------------
c
         call myshift(uwk,jstas,jends)
         call myshift(wwk,jstas,jends)
c
         do j = jsta2,jend1
            do i = 3,iax-1
               uhat(i,j) =
     $              (apx1(1,i,j)*uwk(i,j-1)+apx1(2,i,j)*uwk(i-1,j)
     $              +apx1(4,i,j)*uwk(i+1,j)+apx1(5,i,j)*uwk(i,j+1)
     $              +vgx(i,j+1)*(wwk(i,j+1)-wwk(i-1,j+1))
     $              -vgx(i,j)*(wwk(i,j)-wwk(i-1,j)))/apx1(3,i,j)
            enddo
         enddo
         do j = jsta2,jend1
            uhat(2,j) = 0.d0
            uhat(iax,j) = 0.d0
         enddo
c
         do j = jsta3,jend1
            do i = 2,iax-1
               what(i,j) =
     $              (apz1(1,i,j)*wwk(i,j-1)+apz1(2,i,j)*wwk(i-1,j)
     $              +apz1(4,i,j)*wwk(i+1,j)+apz1(5,i,j)*wwk(i,j+1)
     $              +vgz(i+1,j)*(uwk(i+1,j)-uwk(i+1,j-1))
     $              -vgz(i,j)*(uwk(i,j)-uwk(i,j-1))
     $              +buoy(i,j))/apz1(3,i,j)
            enddo
         enddo
         if (myrank.eq.0) then
            do i = 2,iax-1
               what(i,2) = 0.d0
            enddo
         endif
         if (myrank.eq.nprocs-1) then
            do i = 2,iax-1
               what(i,jax) = 0.d0
            enddo
         endif
c
c ------------------------------------
c     step 3: guess pressure field
c ------------------------------------
c
         call mypartialshift(what,jsta,jend,2)
         do j = jsta2,jend1
            do i = 2,iax-1
               bvcp(i,j) = -dz(j)*(uhat(i+1,j)-uhat(i,j))
     $              -dx(i)*(what(i,j+1)-what(i,j))
               xvcp(i,j) = pstar(i,j)
            enddo
         enddo
         eitr = epsres
         nitr = nmp
         if (myrank.eq.0) bvcp(2,2) = 0.d0
         call ssorcg2p(amtp,bvcp,xvcp,wkrp,wkpp,wkrup,wkapp,mxp,mzp
     $        ,eitr,nitr
     $        ,is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2,
     $        jstap,jendp)
c         if (myrank.eq.0) then
c            write(*,'("solving pstar")')
c            write(*,'(2a11)')"nitr","eitr"
c            write(*,'(i11,e11.4)')nitr,eitr
c         endif
c
         do j = jsta2,jend1
            do i = 2,iax-1
               pstar(i,j) =xvcp(i,j)*arelp+pstar(i,j)*arelp1
            enddo
         enddo
c
c ------------------------------------
c     step 4: guess velocity field
c ------------------------------------
c
         do j = jsta2,jend1
            do i = 3,iax-1
               bvcx(i-1,j) = vgx(i,j+1)*(wwk(i,j+1)-wwk(i-1,j+1))
     $              -vgx(i,j)*(wwk(i,j)-wwk(i-1,j))
     $              -dz(j)*(pstar(i,j)-pstar(i-1,j))
               xvcx(i-1,j) = uwk(i,j)
            enddo
         enddo
         eitr = epsres
         nitr = nmx
         call ssorcg2p(amtx,bvcx,xvcx,wkrx,wkpx,wkrux,wkapx,mxx,mzx
     $        ,eitr,nitr
     $        ,is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2,
     $        jstau,jendu)
c         if (myrank.eq.0) then
c            write(*,'("solving ustar")')
c            write(*,'(2a11)')"nitr","eitr"
c            write(*,'(i11,e11.4)')nitr,eitr
c         endif
c
         do j = jsta2,jend1
            do i = 3,iax-1
               ustar(i,j) = xvcx(i-1,j)*arelu + ustar(i,j)*arelu1
            enddo
         enddo
         do j = jsta2,jend1
            ustar(2,j) = 0.d0
            ustar(iax,j) = 0.d0
         enddo
c
         call mypartialshift(pstar,jsta,jend,1)
         do j = jsta3,jend1
            do i = 2,iax-1
               bvcz(i,j-1) = vgz(i+1,j)*(uwk(i+1,j)-uwk(i+1,j-1))
     $              -vgz(i,j)*(uwk(i,j)-uwk(i,j-1))
     $              -dx(i)*(pstar(i,j)-pstar(i,j-1))
     $              +buoy(i,j)
               xvcz(i,j-1) = wwk(i,j)
            enddo
         enddo
         eitr = epsres
         nitr = nmz
         call ssorcg2p(amtz,bvcz,xvcz,wkrz,wkpz,wkruz,wkapz,mxz,mzz
     $        ,eitr,nitr
     $        ,is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2,
     $        jstaw,jendw)
c         if (myrank.eq.0) then
c            write(*,'("solving wstar")')
c            write(*,'(2a11)')"nitr","eitr"
c            write(*,'(i11,e11.4)')nitr,eitr
c         endif
c
         do j = jsta3,jend1
            do i = 2,iax-1
               wstar(i,j) = xvcz(i,j-1)*arelw + wstar(i,j)*arelw1
            enddo
         enddo
         if (myrank.eq.0) then
            do i = 2,iax-1
               wstar(i,2) = 0.d0
            enddo
         endif
         if (myrank.eq.nprocs-1) then
            do i = 2,iax-1
               wstar(i,jax) = 0.d0
            enddo
         endif
c
c --------------------------------
c     step 5: correct pressure
c --------------------------------
c
         call mypartialshift(wstar,jsta,jend,2)
         do j = jsta2,jend1
            do i = 2,iax-1
               bvcp(i,j) = -dz(j)*(ustar(i+1,j)-ustar(i,j))
     $              -dx(i)*(wstar(i,j+1)-wstar(i,j))
               xvcp(i,j) = 0.d0
            enddo
         enddo
c
c     check convergent or not
c
         zansatmp = 0.d0
         do j = jsta2,jend1
            do i = 2,iax-1
               zansatmp = dmax1(zansatmp,dabs(bvcp(i,j)))
            enddo
         enddo
         call mpi_allreduce(zansatmp,zansa,1,mpi_double_precision,
     $        mpi_max,mpi_comm_world,ierr)
         errors(nsmp) = zansa
c         if (myrank.eq.0) write(*,*)nsmp,zansa
         if ((zansa.le.epssim).and.(nsmp.ne.1)) then
            ich = 0
            goto 999
         endif
c
         if (myrank.eq.0) bvcp(2,2) = 0.d0
         eitr = epsres
         nitr = nmp
         call ssorcg2p(amtp,bvcp,xvcp,wkrp,wkpp,wkrup,wkapp,mxp,mzp
     $        ,eitr,nitr
     $        ,is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2,
     $        jstap,jendp)
c         if (myrank.eq.0) then
c            write(*,'("solving pdash")')
c            write(*,'(2a11)')"nitr","eitr"
c            write(*,'(i11,e11.4)')nitr,eitr
c         endif
c
         do j = jsta2,jend1
            do i = 2,iax-1
               pdash(i,j) = xvcp(i,j)
            enddo
         enddo
c
c --------------------------------------
c     step 6: correct velocity field
c --------------------------------------
c
         do j = jsta2,jend1
            do i = 3,iax-1
               udash(i,j) = -dudp1(i,j)*(pdash(i,j)-pdash(i-1,j))
               uwk(i,j) = ustar(i,j) + udash(i,j)
            enddo
         enddo
         do j = jsta2,jend1
            uwk(2,j) = 0.d0
            uwk(iax,j) = 0.d0
         enddo
         if (myrank.eq.0) then
            do i = 1,iax
               uwk(i,1) = uwk(i,2)
            enddo
         endif
         if (myrank.eq.nprocs-1) then
            do i = 1,iax
               uwk(i,jax) = uwk(i,jax-1)
            enddo
         endif
c
         call mypartialshift(pdash,jsta,jend,1)
         do j = jsta3,jend1
            do i = 2,iax-1
               wdash(i,j) = -dwdp1(i,j)*(pdash(i,j)-pdash(i,j-1))
               wwk(i,j) = wstar(i,j) + wdash(i,j)
            enddo
         enddo
         if (myrank.eq.0) then
            do i = 2,iax-1
               wwk(i,2) = 0.d0
            enddo
         endif
         if (myrank.eq.nprocs-1) then
            do i = 2,iax-1
               wwk(i,jax) = 0.d0
            enddo
         endif
c         do j = 1,jax
         do j = jsta,jend
            wwk(1,j) = wwk(2,j)
            wwk(iax,j) = wwk(iax-1,j)
         enddo
c
c     relaxation is applied after 2nd iteration loop
c
      arelu = relu
      arelw = relw
      arelp = relp
      arelu1 = 1.d0-arelu
      arelw1 = 1.d0-arelw
      arelp1 = 1.d0-arelp
c
      enddo
 999  continue
c ////////////////////////
c     simpler loop end
c ////////////////////////
c
      if (myrank.eq.0) then
         if (ich.eq.0) then
            write(*,50)nsmp,zansa
         else
            write(24,51)maxsmp,epssim
c            do i = 1,maxsmp
c               write(24,*)i,errors(i)
c            enddo
c            stop
         endif
      endif
 50   format("OK/SIM ",i4,e11.4)
 51   format("NG/SIM ",i4,e11.4)
c
c ---------------------------------
c     replace array before exit
c ---------------------------------
c
      do j = jsta,jend
         do i = 1,iax
            u(i,j) = uwk(i,j)
            w(i,j) = wwk(i,j)
         enddo
      enddo
c
      do j = jsta2,jend1
         u(2,j) = 0.d0
         u(iax,j) = 0.d0
         w(1,j) = w(2,j)
         w(iax,j) = w(iax-1,j)
      enddo
      if (myrank.eq.0) then
         do i = 1,iax
            u(i,1) = u(i,2)
            w(i,2) = 0.d0
         enddo
      endif
      if (myrank.eq.nprocs-1) then
         do i = 1,iax
            u(i,jax) = u(i,jax-1)
            w(i,jax) = 0.d0
         enddo
      endif
c
      return
      end
