      program main
      use para4common
***********************************************************************
*     mantle convection in two-dimensional rectangular box
*     hydrodynamic equations are solved by SIMPLER method
*     thermal equation are solved by explicit scheme
*     heat flux is defined by power-law scheme
*     originally written by M. Kameyama on 19990629
*
*     ver.1 on 20021119; linear solvers are changed
*                        red-black SOR for temperature equation
*                        SSOR+CG for SIMPLER
*     ver.2.o on 20021120; parallelization start
***********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j
      real*8 u(iax,jax),w(iax,jax)
      real*8 visc(iax,jax),buoy(iax,jax),sch(iax,jax)
      real*8 tmp(iax,jax)
      real*8 rat,dx(iax),dz(jax),dxs(iax),dzs(jax),asp
      real*8 tsurf,tbot,tamp,qbot,vboti
      integer nt,ntime,maxtime,iplot,iprnt,nstep
      real*8 time,dt
      real*8 siv(iax,jax),eiv(iax,jax)
      real*8 sivo(iax,jax),eivo(iax,jax),viso(iax,jax)
      real*8 xp(iax),zp(jax),xc(iax),zc(jax)
      integer ivlt,icont,ipt,ivis
      real*8 etl,etn,epl,epn
      real*8 dxi(iax),dxsi(iax),dzi(jax),dzsi(jax),areai(iax,jax)
      real*8 cx1(iax),cx2(iax),cx3(iax),cx4(iax)
      real*8 cz1(jax),cz2(jax),cz3(jax),cz4(jax)
      real*8 relu,relw,relp,crmax
      character*80 ipf,opf,flog,file9,file10,file11,file20
      parameter(ipf="fort.21",opf="fort.22",flog="fort.24"
     $     ,file9="fort.9",file10="fort.10",file11="fort.11"
     $     ,file20="fort.20")
c
c     for parallelization
c
      integer jsta,jend,jstas(0:maxprocs-1),jends(0:maxprocs-1)
      integer igather,igather1,igather2
c
c ----------------------
      call mympiinit
c ----------------------
c
c ------------------------------------
c     split domains in z-direction
c ------------------------------------
c
      call para_range(1,jax,jstas,jends)
      jsta = jstas(myrank)
      jend = jends(myrank)
c
      open(24,file=flog)
      call rdata(maxtime,nstep,iplot,iprnt,time,ivlt,icont,ivis,
     $     ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,ipf,
     $     relu,relw,relp,crmax)
      call dfmesh(dx,dz,dxs,dzs,asp,xp,zp,xc,zc)
      call mconst(dx,dxs,dz,dzs,xp,xc,zp,zc
     $     ,dxi,dxsi,dzi,dzsi,areai
     $     ,cx1,cx2,cx3,cx4,cz1,cz2,cz3,cz4)
      call init(asp,dx,dz,dxs,dzs,xp,zp,xc,zc,tmp,sch
     $     ,tsurf,tbot,tamp,siv,eiv,visc,u,w,icont,file9)
c
c ==========================
c     begin time marching
c ==========================
c
      open(11,file=file11)
      open(20,file=file20)
      do nt = 1,maxtime
         igather = 1
         ntime = nt + nstep
         do j = 1,jax
            do i = 1,iax
               eivo(i,j) = eiv(i,j)
               sivo(i,j) = siv(i,j)
               viso(i,j) = visc(i,j)
            enddo
         enddo
c
c ----------------------------------------------
c     calculate buoyancy term at mesh points
c ----------------------------------------------
c
         call buoyant(tmp,rat,buoy,jstas,jends)
c
c -----------------------------------
c     solve hydrodynamic equation
c -----------------------------------
c
         call calvisc(tmp,zp,etl,epl,visc,jstas,jends)
         call hdesim2(u,w,visc,dx,dz,dxs,dzs,dxi,dzi,dxsi,dzsi,buoy,
     $        relu,relw,relp,jstas,jends)
         call caleiv(u,w,eiv,siv,visc
     $        ,dx,dz,dxs,dzs,dxi,dzi,dxsi,dzsi,jstas,jends)
c
c --------------------------
c     evolve temperature
c --------------------------
c
         call tstep(u,w,dx,dz,dxs,dzs,siv,eiv,dt,crmax,jstas,jends)
         call htrns(tmp,u,w,sch,dx,dz,dxs,dzs,asp,
     $        tsurf,tbot,qbot,vboti,dt,jstas,jends)
c_ooban_begin
c         call myooban(tmp,jstas,jends)
c         call myooban(visc,jstas,jends)
c         call myooban(u,jstas,jends)
c         call myooban(w,jstas,jends)
c         call myooban(eiv,jstas,jends)
c         call myooban(siv,jstas,jends)
c_ooban_end
         time = time + dt
c
c         if (myrank.eq.0) then
c
c     for printing
c
         igather1 = mod(nt,iprnt)
         igather2 = mod(nt,iplot)
         igather = igather1*igather2
         if (igather.eq.0) then
            call mygather(tmp,jstas,jends)
            call mygather(u,jstas,jends)
            call mygather(w,jstas,jends)
            call mygather(eiv,jstas,jends)
            call mygather(siv,jstas,jends)
c
            if (igather1.eq.0) then
               if (myrank.eq.0) then
                  call plot(iax,jax,asp,tmp,tsurf,tbot,qbot,
     $                 dx,dz,u,w,eiv,siv,time,ntime)
               endif
            endif
c
c     for drawing
c
            if (igather2.eq.0) then
               call mygather(sch,jstas,jends)
               call mygather(visc,jstas,jends)
               if (myrank.eq.0) then
                  write(11,*)ntime,time
                  write(11,511)tmp,u,w,sch,siv,eiv,visc
                  call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
               endif
            endif
c
         endif
c
      enddo
c
c =========================
c     end time marching
c =========================
c
      call mygather(tmp,jstas,jends)
      call mygather(u,jstas,jends)
      call mygather(w,jstas,jends)
      call mygather(eiv,jstas,jends)
      call mygather(siv,jstas,jends)
      call mygather(sch,jstas,jends)
      call mygather(visc,jstas,jends)
      if (myrank.eq.0) then
c
c ----------------------
c     output results
c ----------------------
c
c     benckmark comparison
c
         call bcheck2d(tmp,u,w,xp,zp,dx,dz,dxs,dzs,asp)
c
c     for a next run
c
         call wdata(maxtime,nstep,iplot,iprnt,time,ivlt,icont,ivis,
     $        ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,opf,
     $        relu,relw,relp,crmax)
         open(10,file=file10,form="unformatted")
         write(10)tmp,u,w,sch,siv,eiv,visc
         close(10)
         if (mod(ntime,iplot).ne.0) then
            write(11,*)ntime,time
            write(11,511)tmp,u,w,sch,siv,eiv,visc
            call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
         endif
         if (mod(nt-1,iprnt).ne.0) then
            call plot(iax,jax,asp,tmp,tsurf,tbot,qbot,
     $           dx,dz,u,w,eiv,siv,time,ntime-1)
         endif
c
      endif
 511  format(8e11.4)
c
c --------------------------
      call mympifinalize
c --------------------------
c
      stop
      end
