**********************************************************************
*     This file contains setup tools for MPI calculations
*     ver.0 on 20020828
*     ver.1 in 20021120
**********************************************************************
      module para4common
      include 'mpif.h'
      integer ierr,myrank,nprocs,istatus(mpi_status_size)
      integer maxprocs
      parameter(maxprocs=80)
      end module para4common
c
c --------------------------------------------------------------------
c
      subroutine mympiinit
      use para4common
      call mpi_init(ierr)
      call mpi_comm_size(mpi_comm_world,nprocs,ierr)
      call mpi_comm_rank(mpi_comm_world,myrank,ierr)
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine mympifinalize
      use para4common
      call mpi_finalize(ierr)
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine para_range(n1,n2,istas,iends)
      use para4common
**********************************************************************
*     splits do-loop iteration range into NPROCS's processors
*
*     taken from "RS/6000 SP Heiretsu-Programming Tora-no-Maki
*     MPI-ban" provided by IBM-Japan
**********************************************************************
      implicit none
c
      integer istas(0:maxprocs-1),iends(0:maxprocs-1),n1,n2
      integer irank,iwork1,iwork2,iwork3,iwork4
c
      iwork1 = (n2-n1+1)/nprocs
      iwork2 = mod(n2-n1+1,nprocs)
      do irank = 0,nprocs-1
         iwork3 = irank*iwork1+n1+min(irank,iwork2)
         iwork4 = iwork3+iwork1-1
         if (iwork2.gt.irank) iwork4 = iwork4+1
         istas(irank) = iwork3
         iends(irank) = iwork4
      enddo
check_begin
c      write(50+myrank,'("rank",i2," have",2i3)')
c     $     myrank,istas(myrank),iends(myrank)
check_end
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine mygather(x,jstas,jends)
      use para4common
**********************************************************************
*     gather distributed data to process 0
**********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,jsta,jend
      integer jstas(0:maxprocs-1),jends(0:maxprocs-1)
      real*8 x(iax,jax),y(iax,jax)
      integer jdisp(0:maxprocs-1),jjlen(0:maxprocs-1)
      integer irank,iireq(0:maxprocs-1),ireq
c
      do irank = 0,nprocs-1
         jjlen(irank) = iax*(jends(irank)-jstas(irank)+1)
c         jdisp(irank) = iax*(jstas(irank)-1)
      enddo
c
      if (myrank.eq.0) then
         do irank = 1,nprocs-1
            call mpi_irecv(x(1,jstas(irank)),jjlen(irank),
     $           mpi_double_precision,
     $           irank,1,mpi_comm_world,iireq(irank),ierr)
         enddo
         do irank = 1,nprocs-1
            call mpi_wait(iireq(irank),istatus,ierr)
         enddo
c
      else
         call mpi_isend(x(1,jstas(myrank)),jjlen(myrank),
     $        mpi_double_precision,
     $        0,1,mpi_comm_world,ireq,ierr)
         call mpi_wait(ireq,istatus,ierr)
      endif
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine myooban(x,jstas,jends)
      use para4common
**********************************************************************
**********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,jsta,jend
      integer jstas(0:maxprocs-1),jends(0:maxprocs-1)
      real*8 x(iax,jax),y(iax,jax)
      integer jdisp(0:maxprocs-1),jjlen(0:maxprocs-1)
      integer irank
c
      do irank = 0,maxprocs-1
         jjlen(irank) = 0
         jdisp(irank) = 0
      enddo
      do irank = 0,nprocs-1
         jjlen(irank) = iax*(jends(irank)-jstas(irank)+1)
         jdisp(irank) = iax*(jstas(irank)-1)
      enddo
c
check_begin
c      do j = 1,jax
c         do i = 1,iax
c            y(i,j) = x(i,j)
c         enddo
c      enddo
check_end
      do irank = 0,nprocs-1
         call mpi_bcast(x(1,jstas(irank)),jjlen(irank),
     $        mpi_double_precision,
     $        irank,mpi_comm_world,ierr)
      enddo
check_begin
c      write(50+myrank,*)"in OOBAN at rank",myrank
c      write(50+myrank,'(2a3,2a12)')"i","j","before","after"
c      do j = 1,jax
c         do i = 1,iax
c            write(50+myrank,'(2i3,2e12.4)')i,j,y(i,j),x(i,j)
c         enddo
c      enddo
check_end
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine myshift(x,jstas,jends)
      use para4common
**********************************************************************
**********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,jsta,jend
      integer jstas(0:maxprocs-1),jends(0:maxprocs-1)
      real*8 x(iax,jax),y(iax,jax)
      integer jdisp(0:maxprocs-1),jjlen(0:maxprocs-1)
      integer irank,iup,idown
      integer ireqs1,ireqs2,ireqr1,ireqr2
c
c ------------------------------------------
c     define communication configuration
c ------------------------------------------
c
      jsta = jstas(myrank)
      jend = jends(myrank)
      iup = myrank+1
      idown = myrank-1
      if (iup.eq.nprocs) iup = mpi_proc_null
      if (idown.eq.-1) idown = mpi_proc_null
c
c --------------------------------
c     shift-type communication
c --------------------------------
c
check_begin
c      do j = max(1,jsta-1),min(jax,jend+1)
c         do i = 1,iax
c            y(i,j) = x(i,j)
c         enddo
c      enddo
check_end
      call mpi_isend(x(1,jend),iax,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqs1,ierr)
      call mpi_isend(x(1,jsta),iax,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqs2,ierr)
      call mpi_irecv(x(1,jsta-1),iax,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqr1,ierr)
      call mpi_irecv(x(1,jend+1),iax,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqr2,ierr)
      call mpi_wait(ireqs1,istatus,ierr)
      call mpi_wait(ireqs2,istatus,ierr)
      call mpi_wait(ireqr1,istatus,ierr)
      call mpi_wait(ireqr2,istatus,ierr)
check_begin
c      write(50+myrank,*)"in SHIFT at rank",myrank
c      write(50+myrank,'(2a3,2a12)')"i","j","before","after"
c      do j = max(1,jsta-1),min(jax,jend+1)
c         do i = 1,iax
c            write(50+myrank,'(2i3,2e12.4)')i,j,y(i,j),x(i,j)
c         enddo
c      enddo
check_end
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine mypartialshift(x,jsta,jend,iflag)
      use para4common
**********************************************************************
**********************************************************************
      implicit none
      include "meshnum"
c
      integer jsta,jend,jsta1,jend1,iflag
      integer iup,idown,ireqs,ireqr
      real*8 x(iax,jax)
c
      iup = myrank+1
      idown = myrank-1
      if (iup.eq.nprocs) iup = mpi_proc_null
      if (idown.eq.-1) idown = mpi_proc_null
c
      if (iflag.eq.1) then
c
c     retrieve from idown
c
         jsta1 = jsta-1
         jend1 = jend
c         if (myrank.eq.0) jsta1 = jsta
         call mpi_isend(x(1,jend1),iax,mpi_double_precision,iup,
     $        1,mpi_comm_world,ireqs,ierr)
         call mpi_irecv(x(1,jsta1),iax,mpi_double_precision,idown,
     $        1,mpi_comm_world,ireqr,ierr)
         call mpi_wait(ireqs,istatus,ierr)
         call mpi_wait(ireqr,istatus,ierr)
      else if (iflag.eq.2) then
c
c     retrieve from iup
c
         jsta1 = jsta
         jend1 = jend+1
c         if (myrank.eq.nprocs-1) jend1 = jend
         call mpi_isend(x(1,jsta1),iax,mpi_double_precision,idown,
     $        1,mpi_comm_world,ireqs,ierr)
         call mpi_irecv(x(1,jend1),iax,mpi_double_precision,iup,
     $        1,mpi_comm_world,ireqr,ierr)
         call mpi_wait(ireqs,istatus,ierr)
         call mpi_wait(ireqr,istatus,ierr)
      endif
c
      return
      end
