      subroutine sor2c2p(a,b,x,relc,istat,nitr,eitr,jstas,jends)
      use para4common
**********************************************************************
*     solve linear simultaneous equations (in 2-D FDM)
*     by Successive Over-Relaxation method with 2 coloring
*     ver.0 on 20021129; MPI parallelization
*
*     variables:
*       a: non-zero elements of coefficients
*       b: elements of right-hand side vector
*       x: initial guess on input, solution on output
*       relc: relaxation coefficient
*       istat: exit status; 0 for normal exit, 1 for abnormal exit
*       nitr: maximun iteration on input, actual iteration on output
*       eitr: maximum torrelance on input, residual on output
**********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,jsta,jend
      real*8 a(5,iax,jax),b(iax,jax),x(iax,jax)
c
      integer maxsor,nsor,istat,nitr
      real*8 p(iax,jax),a3i(iax,jax)
      real*8 eitr,eps,relc,eitrtmp
      real*8 zansa,arelax(iax,jax)
      integer jstas(0:maxprocs-1),jends(0:maxprocs-1)
c
c     split domains in z-direction
c
      jsta = max(2,jstas(myrank))
      jend = min(jax-1,jends(myrank))
c
c     initial guess of solution
c
      do j = jsta,jend
         do i = 2,iax-1
            p(i,j) = x(i,j)
         enddo
      enddo
c
c     relax coeffifient of each point
c
      do j = jsta,jend
         do i = 2,iax-1
            arelax(i,j) = relc
         enddo
      enddo
c
c     maximum torrelance
c
      eps = eitr
      maxsor = nitr
c
c     inverse of diagonal elements
c
      do j = jsta,jend
         do i = 2,iax-1
            a3i(i,j) = 1.d0/a(3,i,j)
         enddo
      enddo
c
c ==================
c     loop start
c ==================
c
      do nsor = 1,maxsor
         eitrtmp = 0.d0
c
         call shift4sor2c2p(p,0,jsta,jend)
c
c     1st color (for even i+j+k)
c
         do j = jsta,jend
            do i = mod(j,2)+2,iax-1,2
               zansa = a3i(i,j)*(
     $              b(i,j)-a(1,i,j)*p(i-1,j)-a(2,i,j)*p(i,j-1)
     $              -a(4,i,j)*p(i,j+1)-a(5,i,j)*p(i+1,j)
     $              )-p(i,j)
               p(i,j) = p(i,j)+arelax(i,j)*zansa
               eitrtmp = dmax1(dabs(zansa),eitrtmp)
            enddo
         enddo
c
         call shift4sor2c2p(p,1,jsta,jend)
c
c     2nd color (for odd i+j+k)
c
         do j = jsta,jend
            do i = mod(j+1,2)+2,iax-1,2
               zansa = a3i(i,j)*(
     $              b(i,j)-a(1,i,j)*p(i-1,j)-a(2,i,j)*p(i,j-1)
     $              -a(4,i,j)*p(i,j+1)-a(5,i,j)*p(i+1,j)
     $              )-p(i,j)
               p(i,j) = p(i,j)+arelax(i,j)*zansa
               eitrtmp = dmax1(dabs(zansa),eitrtmp)
c               if (dabs(zansa).gt.eitrtmp) then
c                  ix = i
c                  jx = j
c                  eitrtmp = dabs(zansa)
c               endif
            enddo
         enddo
c
         call mpi_allreduce(eitrtmp,eitr,1,mpi_double_precision,mpi_max,
     $        mpi_comm_world,ierr)
c
         if (eitr.lt.eps) then
            istat = 0
            nitr = nsor
            goto 999
         endif
      enddo
      istat = 1
      nitr = maxsor
 999  continue
c
c ================
c     loop end
c ================
c
c ===========================
c     replace arrays p->x
c ===========================
c
      do j = jsta,jend
         do i = 2,iax-1
            x(i,j) = p(i,j)
         enddo
      enddo
c
      return
      end
c
c -------------------------------------------------------------------
c
      subroutine shift4sor2c2p(p,iflag,jsta,jend)
      use para4common
**********************************************************************
*     shift data between processes (n) and (n-1)
*     and between process (n) and (n+1)
*
*     iflg: = 0 when called before calculating 1st color
*           = 1 when called before calculating 2nd color
**********************************************************************
      implicit none
      include "meshnum"
c
      real*8 p(iax,jax)
      integer i,j,icnt1,icnt2,ireqs1,ireqs2,ireqr1,ireqr2
      integer iflag
      integer nbuf,jsta,jend,idown,iup
      parameter(nbuf=nx)
      real*8 bufs1(nbuf),bufs2(nbuf),bufr1(nbuf),bufr2(nbuf)
      integer is1,is2,ir1,ir2
c
c     index for receiving buffer
c
      ir1 = mod(jsta+iflag,2)+2
      ir2 = mod(jend+iflag,2)+2
c
c     index for sending buffer
c
      is1 = 5-ir1
      is2 = 5-ir2
c
c     fill sending buffer from (n) to (n-1)
c
      if (myrank.ne.0) then
         j = jsta
         icnt1 = 0
         do i = is1,iax-1,2
            icnt1 = icnt1+1
            bufs1(icnt1) = p(i,j)
         enddo
      endif
c
c     fill sending buffer from (n) to (n+1)
c
      if (myrank.ne.nprocs-1) then
         j = jend
         icnt2 = 0
         do i = is2,iax-1,2
            icnt2 = icnt2+1
            bufs2(icnt2) = p(i,j)
         enddo
      endif
c
c     define destination/origin of data transfer
c
      idown = myrank-1
      iup = myrank+1
      if (idown.eq.-1) idown = mpi_proc_null
      if (iup.eq.nprocs) iup = mpi_proc_null
c
c     send data
c
      call mpi_isend(bufs1,icnt1,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqs1,ierr)
      call mpi_isend(bufs2,icnt2,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqs2,ierr)
c
c     receive data
c
      call mpi_irecv(bufr1,nbuf,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqr1,ierr)
      call mpi_irecv(bufr2,nbuf,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqr2,ierr)
c
c     block
c
      call mpi_wait(ireqs1,istatus,ierr)
      call mpi_wait(ireqs2,istatus,ierr)
      call mpi_wait(ireqr1,istatus,ierr)
      call mpi_wait(ireqr2,istatus,ierr)
c
c     retrieve from receiving buffer
c
      if (myrank.ne.0) then
         j = jsta-1
         icnt1 = 0
         do i = ir1,iax-1,2
            icnt1 = icnt1+1
            p(i,j) = bufr1(icnt1)
         enddo
      endif
c
      if (myrank.ne.nprocs-1) then
         j = jend+1
         icnt2 = 0
         do i = ir2,iax-1,2
            icnt2 = icnt2+1
            p(i,j) = bufr2(icnt2)
         enddo
      endif
c
      return
      end
