      subroutine ssorcg2p(a,b,x,r,p,ru,ap,mx,mz,eps,nitr
     $     ,is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2,jstas,jends)
      use para4common
**********************************************************************
*     conjugate gradient method for SSOR-CG
*     solve 2-D Poisson equation by FDM
*
*     ver.0 on 20021119
*     ver.1 on 20021129; MPI parallelization
*
*     global variables:
*       mx,mz: number of solution points in each direction
*       a(5,mx+2,mz+2): nonzero elements of coefficient matrix
*       b(mx+2,mz+2): RHS vector
*       x(mx+2,mz+2): initial guess on input, solution on output
*       eps: tolerance on input, truncation error on output
*       nitr: maximum iteration on input, iteration number on output
*       r(mx+2,mz+2):
*       p(mx+2,mz+2):
*       ru(mx+2,mz+2):
*       ap(mx+2,mz+2):
*     arrays for message passing
*       is1,is2,ir1,ir2,bufs1,bufs2,bufr1,bufr2
**********************************************************************
      implicit none
c
      integer i,j,ncg,maxitr
      integer mx,mz,nitr
      real*8 a(5,mx+2,mz+2),b(mx+2,mz+2),x(mx+2,mz+2)
      real*8 r(mx+2,mz+2),p(mx+2,mz+2),ru(mx+2,mz+2)
     $     ,ap(mx+2,mz+2)
      real*8 alpha,b2,beta,eps,eps2
      real*8 rur0,rur1,pap,res2
c
      integer jsta,jend,jstas(0:maxprocs-1),jends(0:maxprocs-1)
      integer is1,is2,ir1,ir2
      real*8 bufs1(mx+2),bufs2(mx+2),bufr1(mx+2),bufr2(mx+2)
c
      maxitr = nitr
      eps2 = eps**2
c
c     split domains in z-direction
c
      jsta = max(2,jstas(myrank))
      jend = min(mz+1,jends(myrank))
c
c ----------------------
c     initialization
c ----------------------
c
      call prodmv2p(a,x,ap,mx,mz,jsta,jend)
c
      do j = jsta,jend
         do i = 2,mx+1
            r(i,j) = b(i,j)-ap(i,j)
         enddo
      enddo
c
      call ssorsl2p(a,r,ru,mx,mz,jsta,jend,is1,is2,ir1,ir2
     $     ,bufs1,bufs2,bufr1,bufr2)
c
      do j = jsta,jend
         do i = 2,mx+1
            p(i,j) = ru(i,j)
         enddo
      enddo
      call prodvv2p(r,ru,rur0,mx,mz,jsta,jend)
c
      call prodvv2p(b,b,b2,mx,mz,jsta,jend)
c
c ============================
c     iterative loop start
c ============================
c
      do ncg = 1,maxitr
c
         call prodmv2p(a,p,ap,mx,mz,jsta,jend)
c
         call prodvv2p(p,ap,pap,mx,mz,jsta,jend)
c
         alpha = rur0/pap
         do j = jsta,jend
            do i = 2,mx+1
               x(i,j) = x(i,j)+alpha*p(i,j)
               r(i,j) = r(i,j)-alpha*ap(i,j)
            enddo
         enddo
         call prodvv2p(r,r,res2,mx,mz,jsta,jend)
c
         eps = dsqrt(res2/b2)
         if (res2/b2.le.eps2) then
c
c -----------------
c     converged
c -----------------
c
            nitr = ncg
            goto 1000
         endif
c
c ---------------------
c     not converged
c ---------------------
c
         call ssorsl2p(a,r,ru,mx,mz,jsta,jend,is1,is2,ir1,ir2
     $        ,bufs1,bufs2,bufr1,bufr2)
c
         call prodvv2p(r,ru,rur1,mx,mz,jsta,jend)
         beta = rur1/rur0
         rur0 = rur1
c
         do j = jsta,jend
            do i = 2,mx+1
               p(i,j) = ru(i,j)+beta*p(i,j)
            enddo
         enddo
c
c ==========================
c     iterative loop end
c ==========================
c
      enddo
c
 1000 continue
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine prodmv2p(a,x,ap,mx,mz,jsta,jend)
      use para4common
**********************************************************************
*     Compute product
*     AP = (matrix A) * (vector X)
**********************************************************************
      implicit none
c
      integer i,j
      integer mx,mz
      real*8 a(5,mx+2,mz+2),x(mx+2,mz+2),ap(mx+2,mz+2)
c
      integer jsta,jend,idown,iup
      integer ndata,ireqs1,ireqs2,ireqr1,ireqr2
c
c     message passing
c
      ndata = mx+2
      idown = myrank-1
      iup = myrank+1
      if (idown.eq.-1) idown = mpi_proc_null
      if (iup.eq.nprocs) iup = mpi_proc_null
      call mpi_isend(x(1,jsta),ndata,mpi_double_precision,idown,1
     $     ,mpi_comm_world,ireqs1,ierr)
      call mpi_isend(x(1,jend),ndata,mpi_double_precision,iup,1
     $     ,mpi_comm_world,ireqs2,ierr)
      call mpi_irecv(x(1,jsta-1),ndata,mpi_double_precision,idown,1
     $     ,mpi_comm_world,ireqr1,ierr)
      call mpi_irecv(x(1,jend+1),ndata,mpi_double_precision,iup,1
     $     ,mpi_comm_world,ireqr2,ierr)
      call mpi_wait(ireqs1,istatus,ierr)
      call mpi_wait(ireqs2,istatus,ierr)
      call mpi_wait(ireqr1,istatus,ierr)
      call mpi_wait(ireqr2,istatus,ierr)
c
      do j = jsta,jend
         do i = 2,mx+1
            ap(i,j) = a(1,i,j)*x(i,j-1)+a(2,i,j)*x(i-1,j)
     $           +a(3,i,j)*x(i,j)+a(4,i,j)*x(i+1,j)+a(5,i,j)*x(i,j+1)
         enddo
      enddo
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine prodvv2p(vec1,vec2,vdotv,mx,mz,jsta,jend)
      use para4common
**********************************************************************
*     Compute dot product of two vectors
*     vdotv = (vec1,vec2)
**********************************************************************
      implicit none
c
      integer i,j
      integer mx,mz
      real*8 vec1(mx+2,mz+2),vec2(mx+2,mz+2),vdotv
c
      integer jsta,jend
      real*8 vdotvtmp
c
      vdotvtmp = 0.d0
      do j = jsta,jend
         do i = 2,mx+1
            vdotvtmp = vdotvtmp+vec1(i,j)*vec2(i,j)
         enddo
      enddo
c
      call mpi_allreduce(vdotvtmp,vdotv,1,mpi_double_precision,mpi_sum
     $     ,mpi_comm_world,ierr)
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine ssorsl2p(a,b,x,mx,mz,jsta,jend,is1,is2,ir1,ir2
     $     ,bufs1,bufs2,bufr1,bufr2)
      use para4common
**********************************************************************
*     preconditioner for CG solving 3-D Poisson equation by FDM
*     Symmetric Successive Over Relaxation method by 2 coloring
*
*     ver.0 on 20020823; as stand-alone SSOR solver for Ax=b
*
*     global variables:
*       a: non-zero elements of coefficients
*       b: elements of right-hand side vector
*       x: initial guess on input, solution on output
*       mx,mz: number of solution points in each direction
*     local variables
*       maxsor: maximun iteration number
*       relc: relaxation coefficient
**********************************************************************
      implicit none
c
      integer i,j
      integer mx,mz
      real*8 a(5,mx+2,mz+2),b(mx+2,mz+2),x(mx+2,mz+2)
c
      integer maxsor,nsor
      parameter(maxsor=1)
      real*8 eitr,relc,zansa
      parameter(relc=1.d0)
      integer jsta,jend
      integer is1,is2,ir1,ir2
      real*8 bufs1(mx+2),bufs2(mx+2),bufr1(mx+2),bufr2(mx+2)
c
c     initial guess of solution
c
      do j = jsta,jend
         do i = 2,mx+1
            x(i,j) = 0.d0
         enddo
      enddo
c
c ==================
c     loop start
c ==================
c
      call shift4ssorsl2p(mx,mz,x,is1,is2,ir1,ir2
     $     ,bufs1,bufs2,bufr1,bufr2,jsta,jend,0)
c
      do nsor = 1,maxsor
c
c +++++++++++++++++++
c     forward SOR
c +++++++++++++++++++
c
c     1st color (for even i+j)
c
         do j = jsta,jend
            do i = mod(j,2)+2,mx+1,2
               zansa = (b(i,j)
     $              -a(1,i,j)*x(i,j-1)-a(2,i,j)*x(i-1,j)
     $              -a(4,i,j)*x(i+1,j)-a(5,i,j)*x(i,j+1)
     $              )/a(3,i,j)-x(i,j)
               x(i,j) = x(i,j)+relc*zansa
            enddo
         enddo
c
c     2nd color (for odd i+j)
c
         call shift4ssorsl2p(mx,mz,x,is1,is2,ir1,ir2
     $        ,bufs1,bufs2,bufr1,bufr2,jsta,jend,1)
         do j = jsta,jend
            do i = mod(j+1,2)+2,mx+1,2
               zansa = (b(i,j)
     $              -a(1,i,j)*x(i,j-1)-a(2,i,j)*x(i-1,j)
     $              -a(4,i,j)*x(i+1,j)-a(5,i,j)*x(i,j+1)
     $              )/a(3,i,j)-x(i,j)
               x(i,j) = x(i,j)+relc*zansa
            enddo
         enddo
c
c ++++++++++++++++++++
c     backward SOR
c ++++++++++++++++++++
c
c     2nd color (for odd i+j)
c
         do j = jend,jsta,-1
            do i = mx+mod(mx+1+j,2),2,-2
               zansa = (b(i,j)
     $              -a(1,i,j)*x(i,j-1)-a(2,i,j)*x(i-1,j)
     $              -a(4,i,j)*x(i+1,j)-a(5,i,j)*x(i,j+1)
     $              )/a(3,i,j)-x(i,j)
               x(i,j) = x(i,j)+relc*zansa
            enddo
         enddo
c
c     1st color (for even i+j)
c
         call shift4ssorsl2p(mx,mz,x,is1,is2,ir1,ir2
     $        ,bufs1,bufs2,bufr1,bufr2,jsta,jend,0)
         do j = jend,jsta,-1
            do i = mx+mod(mx+j,2),2,-2
               zansa = (b(i,j)
     $              -a(1,i,j)*x(i,j-1)-a(2,i,j)*x(i-1,j)
     $              -a(4,i,j)*x(i+1,j)-a(5,i,j)*x(i,j+1)
     $              )/a(3,i,j)-x(i,j)
               x(i,j) = x(i,j)+relc*zansa
            enddo
         enddo
c
      enddo
c
c ================
c     loop end
c ================
c
      return
      end
c
c --------------------------------------------------------------------
c
      subroutine shift4ssorsl2p(mx,mz,p,is1,is2,ir1,ir2
     $     ,bufs1,bufs2,bufr1,bufr2,jsta,jend,iflag)
      use para4common
**********************************************************************
*     shift data between processes (n) and (n-1)
*     and between process (n) and (n+1)
*
*     iflg: = 0 when called before calculating 1st color
*           = 1 when called before calculating 2nd color
**********************************************************************
      implicit none
c
      integer mx,mz
      real*8 p(mx+2,mz+2)
      integer i,j,k,icnt1,icnt2,ireqs1,ireqs2,ireqr1,ireqr2
      integer iflag,jsta,jend,iup,idown
      integer is1,is2,ir1,ir2
      real*8 bufs1(mx+2),bufs2(mx+2),
     $     bufr1(mx+2),bufr2(mx+2)
c
c     index for receiving buffer
c
      ir1 = mod(jsta+iflag,2)+2
      ir2 = mod(jend+iflag,2)+2
c
c     index for sending buffer
c
      is1 = 5-ir1
      is2 = 5-ir2
c
c     fill sending buffer from (n) to (n-1)
c
      if (myrank.ne.0) then
         j = jsta
         icnt1 = 0
         do i = is1,mx+1,2
            icnt1 = icnt1+1
            bufs1(icnt1) = p(i,j)
         enddo
      endif
c
c     fill sending buffer from (n) to (n+1)
c
      if (myrank.ne.nprocs-1) then
         j = jend
         icnt2 = 0
         do i = is2,mx+1,2
            icnt2 = icnt2+1
            bufs2(icnt2) = p(i,j)
         enddo
      endif
c
c     define destination/origin of data transfer
c
      idown = myrank-1
      iup = myrank+1
      if (idown.eq.-1) idown = mpi_proc_null
      if (iup.eq.nprocs) iup = mpi_proc_null
c
c     send data
c
      call mpi_isend(bufs1,icnt1,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqs1,ierr)
      call mpi_isend(bufs2,icnt2,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqs2,ierr)
c
c     receive data
c
      call mpi_irecv(bufr1,mx,mpi_double_precision,idown,
     $     1,mpi_comm_world,ireqr1,ierr)
      call mpi_irecv(bufr2,mx,mpi_double_precision,iup,
     $     1,mpi_comm_world,ireqr2,ierr)
c
c     block
c
      call mpi_wait(ireqs1,istatus,ierr)
      call mpi_wait(ireqs2,istatus,ierr)
      call mpi_wait(ireqr1,istatus,ierr)
      call mpi_wait(ireqr2,istatus,ierr)
c
c     retrieve from receiving buffer
c
      if (myrank.ne.0) then
         j = jsta-1
         icnt1 = 0
         do i = ir1,mx+1,2
            icnt1 = icnt1+1
            p(i,j) = bufr1(icnt1)
         enddo
      endif
c
      if (myrank.ne.nprocs-1) then
         j = jend+1
         icnt2 = 0
         do i = ir2,mx+1,2
            icnt2 = icnt2+1
            p(i,j) = bufr2(icnt2)
         enddo
      endif
c
      return
      end
