      subroutine init(asp,dx,dz,dxs,dzs,xp,zp,xc,zc,
     $     tmp,q,tsurf,tbot,tamp,siv,eiv,visc,u,w,icont,file9)
***********************************************************************
*     set initial conditions
*
*     icont: = 0 if new run
*            = 1 if successive run
*     ivlt: = 1 if free-slip top surface
*           = 2 if fixed velocity top surface
*     input
*     output
*        tmp: temperature field
*        q: internal heating rate
*        tamp: reference temperature
*        bvlt: -2*(surface velocity)*dz
***********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j
      integer icont,ivlt
      real*8 tmp(iax,jax),q(iax,jax),visc(iax,jax)
      real*8 eiv(iax,jax),siv(iax,jax),u(iax,jax),w(iax,jax)
      real*8 bvlt(iax+1)
      real*8 tsurf,tbot,tamp
      real*8 asp,dx(iax),dz(jax),dxs(iax),dzs(jax)
      real*8 pi,xp(iax),xc(iax),zp(jax),zc(jax)
      real*8 cell,dtbl,zz
      real*8 dum
      real*8 t1,t2,tm
      character*80 file9
c
      if (icont .eq. 0) then
c
c =============================
c     for new run (icont=0)
c =============================
c
c -------------------------------
c     set initial temperature
c -------------------------------
c
         pi = 4.d0*datan(1.d0)
         cell = 1.d0
         tm = 0.5d0*(tsurf+tbot)
         dtbl = .5d0
         do j = 2,jax-1
            zz = zp(j)
            t1 = tsurf+(1.d0-zz)*(tm-tsurf)/dtbl
            t2 = dmax1(tbot+zz*(tm-tbot)/dtbl,tm)
            do i = 2,iax-1
               tmp(i,j) = dmin1(t1,t2)
     $              + tamp*dcos(pi*cell*xp(i)/asp)*dsin(pi*zz)
            enddo
         enddo
c
         do i = 2,iax-1
            tmp(i,1) = 2.d0*tbot - tmp(i,2)
            tmp(i,jax) = 2.d0*tsurf - tmp(i,jax-1)
         enddo
         do j = 1,jax
            tmp(1,j) = tmp(2,j)
            tmp(iax,j) = tmp(iax-1,j)
         enddo
c
c ----------------------------------------
c     set initial heat_production_rate
c ----------------------------------------
c
         do j = 2,jax-1
            do i = 2,iax-1
               q(i,j) = 0.d0
            enddo
         enddo
c
c
c -----------------------------------
c     set initial velocity fields
c -----------------------------------
c
         do j = 1,jax
            do i = 1,iax
               u(i,j) = 0.d0
               w(i,j) = 0.d0
            enddo
         enddo
c
         tamp = tbot - tsurf
c
      else if (icont .eq. 1) then
c
c ================================
c     successive run (icont=1)
c ================================
c
         open(9,file=file9,form="unformatted")
         read(9)tmp,u,w,q,siv,eiv,visc
         close(9)
c
      endif
c
      return
      end
