      program main
***********************************************************************
*     mantle convection in two-dimensional rectangular box
*     hydrodynamic equations are solved by SIMPLER method
*     thermal equation are solved by explicit scheme
*     heat flux is defined by power-law scheme
*     originally written by M. Kameyama on 19990629
***********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j
      real*8 u(iax,jax),w(iax,jax)
      real*8 visc(iax,jax),buoy(iax,jax),sch(iax,jax)
      real*8 tmp(iax,jax)
      real*8 rat,dx(iax),dz(jax),dxs(iax),dzs(jax),asp
      real*8 tsurf,tbot,tamp,qbot,vboti
      integer nt,ntime,maxtime,iplot,iprnt,nstep
      real*8 time,dt
      real*8 siv(iax,jax),eiv(iax,jax)
      real*8 sivo(iax,jax),eivo(iax,jax),viso(iax,jax)
      real*8 xp(iax),zp(jax),xc(iax),zc(jax)
      integer ivlt,icont,ipt,ivis
      real*8 etl,etn,epl,epn
      real*8 dxi(iax),dxsi(iax),dzi(jax),dzsi(jax),areai(iax,jax)
      real*8 cx1(iax),cx2(iax),cx3(iax),cx4(iax)
      real*8 cz1(jax),cz2(jax),cz3(jax),cz4(jax)
      real*8 relu,relw,relp,crmax
      character*80 ipf,opf,flog,file9,file10,file11,file20
      parameter(ipf="fort.21",opf="fort.22",flog="fort.24"
     $     ,file9="fort.9",file10="fort.10",file11="fort.11"
     $     ,file20="fort.20")
c
      open(24,file=flog)
      call rdata(maxtime,nstep,iplot,iprnt,time,ivlt,icont,ivis,
     $     ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,ipf,
     $     relu,relw,relp,crmax)
      call dfmesh(dx,dz,dxs,dzs,asp,xp,zp,xc,zc)
      call mconst(dx,dxs,dz,dzs,xp,xc,zp,zc
     $     ,dxi,dxsi,dzi,dzsi,areai
     $     ,cx1,cx2,cx3,cx4,cz1,cz2,cz3,cz4)
      call init(asp,dx,dz,dxs,dzs,xp,zp,xc,zc,tmp,sch
     $     ,tsurf,tbot,tamp,siv,eiv,visc,u,w,icont,file9)
c
c ==========================
c     begin time marching
c ==========================
c     
      open(11,file=file11)
      open(20,file=file20)
      do nt = 1,maxtime
         ntime = nt + nstep
         do j = 1,jax
            do i = 1,iax
               eivo(i,j) = eiv(i,j)
               sivo(i,j) = siv(i,j)
               viso(i,j) = visc(i,j)
            enddo
         enddo
c
c ----------------------------------------------
c     calculate buoyancy term at mesh points
c ----------------------------------------------
c
         call buoyant(tmp,rat,buoy)
c
c -----------------------------------
c     solve hydrodynamic equation
c -----------------------------------
c
         call calvisc(tmp,zp,etl,epl,visc)
         call hdesim2(u,w,visc,dx,dz,dxs,dzs,dxi,dzi,dxsi,dzsi,buoy,
     $        relu,relw,relp)
         call caleiv(u,w,eiv,siv,visc,dxi,dzi,dxsi,dzsi)
c
c --------------------------
c     evolve temperature
c --------------------------
c
         call tstep(u,w,dx,dz,dxs,dzs,siv,eiv,dt,crmax)
         call htrns(tmp,u,w,sch,dx,dz,dxs,dzs,asp,
     $        tsurf,tbot,qbot,vboti,dt)
         time = time + dt
c
c     for printing
c
         if (mod(nt,iprnt) .eq. 0) then
            call plot(iax,jax,asp,tmp,tsurf,tbot,qbot,
     $           dx,dz,u,w,eiv,siv,time,ntime)
         endif
c
c     for drawing
c
         if (mod(nt,iplot) .eq. 0) then
            write(11,*)ntime,time
            write(11,511)tmp,u,w,sch,siv,eiv,visc
            call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
         endif
c
      enddo
c
c =========================
c     end time marching
c =========================
c
      call wdata(maxtime,nstep,iplot,iprnt,time,ivlt,icont,ivis,
     $     ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,opf,
     $     relu,relw,relp,crmax)
c
c ----------------------
c     output results
c ----------------------
c
c     benckmark comparison
c
      call bcheck2d(tmp,u,w,xp,zp,dx,dz,dxs,dzs,asp)
c
c     for a next run
c
      open(10,file=file10,form="unformatted")
      write(10)tmp,u,w,sch,siv,eiv,visc
      close(10)
      if (mod(ntime,iplot) .ne. 0) then
         write(11,*)ntime,time
         write(11,511)tmp,u,w,sch,siv,eiv,visc
         call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
      endif
      if (mod(nt-1,iprnt) .ne. 0) then
         call plot(iax,jax,asp,tmp,tsurf,tbot,qbot,
     $        dx,dz,u,w,eiv,siv,time,ntime-1)
      endif
 511  format(8e11.4)
c
      stop
      end
