      subroutine sor2(a,b,x,relc,istat,nitr,eitr)
**********************************************************************
*     solve linear simultaneous equations (in 2-D FDM)
*     by Successive Over-Relaxation method with 2 coloring
*     ver.0 on 20021119; red-black SOR is implemented
*
*     variables:
*       a: non-zero elements of coefficients
*       b: elements of right-hand side vector
*       x: initial guess on input, solution on output
*       relc: relaxation coefficient
*       istat: exit status; 0 for normal exit, 1 for abnormal exit
*       nitr: maximun iteration on input, actual iteration on output
*       eitr: maximum torrelance on input, residual on output
**********************************************************************
      implicit none
      include "meshnum"
c
      integer i,j,k
      real*8 a(5,iax,jax),b(iax,jax),x(iax,jax)
c
      integer maxsor,nsor,istat,nitr
      real*8 p(iax,jax),a3i(iax,jax)
      real*8 eitr,eps,relc
      real*8 zansa,arelax(iax,jax)
      integer ix,jx
c
c     initial guess of solution
c
      do j = 1,jax
         do i = 1,iax
            p(i,j) = x(i,j)
         enddo
      enddo
c
c     relax coeffifient of each point
c
      do j = 2,jax-1
         do i = 2,iax-1
            arelax(i,j) = relc
         enddo
      enddo
c
c     maximum torrelance
c
      eps = eitr
      maxsor = nitr
c
c     inverse of diagonal elements
c
      do j = 2,jax-1
         do i = 2,iax-1
            a3i(i,j) = 1.d0/a(3,i,j)
         enddo
      enddo
c
      do nsor = 1,maxsor
         eitr = 0.d0
c
c     1st color (for even i+j)
c
         do j = 2,jax-1
            do i = mod(j,2)+2,iax-1,2
               zansa = a3i(i,j)*(
     $              b(i,j)-a(1,i,j)*p(i-1,j)-a(2,i,j)*p(i,j-1)
     $              -a(4,i,j)*p(i,j+1)-a(5,i,j)*p(i+1,j))-p(i,j)
               p(i,j) = p(i,j)+arelax(i,j)*zansa
               if (dabs(zansa).gt.eitr) then
                  ix = i
                  jx = j
                  eitr = dabs(zansa)
               endif
            enddo
         enddo
c
c     2nd color (for odd i+j)
c
         do j = 2,jax-1
            do i = mod(j+1,2)+2,iax-1,2
               zansa = a3i(i,j)*(
     $              b(i,j)-a(1,i,j)*p(i-1,j)-a(2,i,j)*p(i,j-1)
     $              -a(4,i,j)*p(i,j+1)-a(5,i,j)*p(i+1,j))-p(i,j)
               p(i,j) = p(i,j)+arelax(i,j)*zansa
               if (dabs(zansa).gt.eitr) then
                  ix = i
                  jx = j
                  eitr = dabs(zansa)
               endif
            enddo
         enddo
c
c         if (mod(nsor,100).eq.0) then
c            write(*,'(i5,e11.4,3i5)')nsor,eitr,ix,jx,kx
c         endif
         if (eitr.lt.eps) then
            istat = 0
            nitr = nsor
            goto 999
         endif
      enddo
      istat = 1
      nitr = maxsor
 999  continue
c
c     substitute solution
c
      if (istat.eq.0) then
         do j = 2,jax-1
            do i = 2,iax-1
               x(i,j) = p(i,j)
            enddo
         enddo
      endif
c
      return
      end
