#!/bin/sh
trap 'rm -f ${head}*.grd' 0
myname=`basename $0`
cwd=`pwd`
dir=`basename $cwd`

if [ $# -eq 0 ]; then
echo "Usage: $myname (number to be drawn)"
exit 1
fi

###############################
# default configuration begin #
###############################

n=$1
ps=cc$n.ps

# region to be drawn
xrange="0/3"
yrange="0/1"
xgrid=0.00625
ygrid=${xgrid}

# contour interval
cit=0.1
cif=10
cig=0.5
cis=0.5
cie=0.5
civ=0.5

# gridding command
gcom=xyz2grd

# image size
size=5.4/1.8
xoff=1.0
yoff=8.5
xshift=0
yshift=2.0
direct="-P"

#############################
# default configuration end #
#############################

#################################
# read local configuration file #
#################################

localc=./.${myname}
if [ -f $localc ] ; then
  echo reading $localc
  . $localc
fi

range=${xrange}/${yrange}
grid=${xgrid}/${ygrid}
bline="1t5_5:0"

head=/tmp/$$_
for id in tmp siv eiv vis
do
echo ${id}${n}.dat "=>" ${head}${id}${n}.grd
$gcom -G${head}${id}${n}.grd -I$grid -R/$range ${id}${n}.dat
done

# temperature
grdcontour ${head}tmp${n}.grd -C$cit -R$range -JX$size -Bf0/f0 -X$xoff -Y$yoff -V -K $direct > $ps
pstext -JX$size -R0/1/0/1 -N -O -K << END >> $ps
0.0 1.10 14 0 1 1 ${dir}/${ps} ($range)
0.5 1.02 14 0 1 2 T ($cit)
END
rm -f ${head}tmp${n}.grd

# viscosity
grdcontour ${head}vis${n}.grd -A${civ}f9 -R$range -JX$size -Bf0/f0 -Y-$yshift -O -V -K >> $ps
pstext -JX$size -R0/1/0/1 -N -O -K << END >> $ps
0.5 1.02 14 0 1 2 log(@~h@~) ($civ)
END
rm -f ${head}vis${n}.grd

# stress
grdcontour ${head}siv${n}.grd -A${cis}f9 -R$range -JX$size -Bf0/f0 -Y-$yshift -O -V -K >> $ps
pstext -JX$size -R0/1/0/1 -N -O -K <<END >> $ps
0.5 1.02 14 0 1 2 log(@~t@~) ($cis)
END
rm -f ${head}siv${n}.grd

# strain rate
grdcontour ${head}eiv${n}.grd -A${cie}f9 -R$range -JX$size -Bf0/f0 -Y-$yshift -L-100/100 -V -O -K >> $ps
pstext -JX$size -R0/1/0/1 -N -O -K <<END >> $ps
0.5 1.02 14 0 1 2 log(@~e@~) ($cie)
END
rm -f ${head}eiv${n}.grd

echo showpage >> $ps

if [ $DISPLAY ] ; then
gs -q -dNOKANJI $ps
fi

exit 0
